# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import sys

from hardware.cardiff import cardiff
from hardware.cardiff import compare_sets
from hardware.cardiff import utils as cardiff_utils

from ahc_tools import utils


def print_report(args, facts):
    # The global_params are only used for a single output_dir key.
    # The output_dir key is not currently useful for this use case.
    # We could probably refactor hardware to make it a kwarg, so we don't need
    # to pass an empty dictionary.
    global_params = {}
    # The detail dictionary has three keys (group, category, item). This is
    # only used in the case that we have the print_level set to
    # utils.Levels.DETAIL
    # This is not currently used, but required by the architecture of hardware.
    detail = {'group': '', 'category': '', 'item': ''}
    # We have a different kernel cmdline for each system, so we have to ignore
    # system to get groups that have more than one system.
    ignore_list = 'system'
    # unique_id can either be 'serial' or 'uuid', in virtual environments
    # 'serial' is not reported so we default to 'uuid'
    unique_id = args.unique_id
    # Extract the host list from the data to get the initial list of hosts.
    systems_groups = []
    systems_groups.append(cardiff_utils.get_hosts_list(facts, unique_id))
    # Print the group information
    if args.groups or args.full:
        compare_sets.print_systems_groups(systems_groups)

    # Print the category information
    if args.categories or args.full:
        cardiff.group_systems(global_params, facts, unique_id,
                              systems_groups, ignore_list)

    # Print the outlier information
    if args.outliers or args.full:
        cardiff.compare_performance(facts, unique_id, systems_groups, detail)


def main():
    parser = argparse.ArgumentParser(description='Consume benchmark data '
                                                 'from Ironic.')
    parser.add_argument('-f', '--full', action='store_true',
                        help='Print the full report.')
    parser.add_argument('-g', '--groups', action='store_true',
                        help='Print the groupings by similar hardware.')
    parser.add_argument('-c', '--categories', action='store_true',
                        help='Print the report for each category.')
    parser.add_argument('-o', '--outliers', action='store_true',
                        help='Print the report showing outliers.')
    parser.add_argument('-u', '--unique-id', dest='unique_id',
                        action='store', default='uuid',
                        choices=['uuid', 'serial'],
                        help='Unique key to identify the nodes by.')
    args = parser.parse_args()

    # If we did not pass any print arguments, print the help and exit
    if not (args.groups or args.categories or args.outliers or args.full):
        print("\nYou did not specify anything to print.\n")
        parser.print_help()
        sys.exit(1)

    ironic_client = utils.get_ironic_client()
    nodes = ironic_client.node.list(detail=True)
    facts = [utils.get_facts(node) for node in nodes]

    print_report(args, facts)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import unittest

from ahc_tools import utils

from ironicclient import client as ic_client
from ironicclient.exc import AmbiguousAuthSystem
import mock


class TestGetFacts(unittest.TestCase):
    def test_facts(self):
        node1 = mock.Mock(extra={'edeploy_facts':
                                 [[u'cpu', u'logical_0',
                                   u'bogomips', u'4199.99'],
                                  [u'cpu', u'logical_0',
                                   u'cache_size', u'4096KB']
                                  ]})
        node2 = mock.Mock(extra={'edeploy_facts':
                                 [[u'cpu', u'logical_0',
                                   u'bogomips', u'4098.99'],
                                  [u'cpu', u'logical_0',
                                   u'cache_size', u'4096KB']
                                  ]})
        nodes = [node1, node2]
        expected = [[(u'cpu', u'logical_0', u'bogomips', u'4199.99'),
                     (u'cpu', u'logical_0', u'cache_size', u'4096KB')],
                    [(u'cpu', u'logical_0', u'bogomips', u'4098.99'),
                     (u'cpu', u'logical_0', u'cache_size', u'4096KB')]]
        facts = [utils.get_facts(node) for node in nodes]
        self.assertEqual(expected, facts)

    def test_no_facts(self):
        node = mock.Mock(extra={})
        err_msg = ("You must run introspection on the nodes before "
                   "running this tool.\n")
        self.assertRaisesRegexp(SystemExit, err_msg,
                                utils.get_facts, node)


@mock.patch.object(ic_client, 'get_client', autospec=True,
                   side_effect=AmbiguousAuthSystem)
class TestGetIronicClient(unittest.TestCase):
    def test_no_credentials(self, ic_mock):
        os.environ.clear()
        err_msg = ("Some credentials are missing. The needed environment "
                   "variables are set as follows: OS_PASSWORD=None, "
                   "OS_USERNAME=None, OS_TENANT_NAME=None, "
                   "OS_AUTH_URL=None.\n")
        self.assertRaisesRegexp(SystemExit, err_msg,
                                utils.get_ironic_client)
        self.assertTrue(ic_mock.called)

    def test_password_hidden(self, ic_mock):
        os.environ.clear()
        os.environ['OS_PASSWORD'] = "password"
        err_msg = ("Some credentials are missing. The needed environment "
                   "variables are set as follows: OS_PASSWORD=<hidden>, "
                   "OS_USERNAME=None, OS_TENANT_NAME=None, "
                   "OS_AUTH_URL=None.\n")
        self.assertRaisesRegexp(SystemExit, err_msg,
                                utils.get_ironic_client)
        self.assertTrue(ic_mock.called)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class LoadFailedError(Exception):
    """Failure to load the edeploy state file.

    Attributes:
    o_msg -- original message from the exception that occured
    conf_dir -- directory with the state file used by edeploy
    """

    def __init__(self, o_msg, conf_dir):
        msg = ('Unable to load the state file in %s. '
               'Error was: %s' % (conf_dir, o_msg))
        super(LoadFailedError, self).__init__(msg)


class MatchFailedError(Exception):
    """No matching profiles were found.

    Attributes:
    o_msg -- original message from the exception that occured
    uuid -- uuid of the node that failed to match
    """

    def __init__(self, o_msg, uuid):
        msg = ('Failed to match node uuid: %s. Error was: %s' % (uuid, o_msg))
        super(MatchFailedError, self).__init__(msg)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

EDEPLOY_OPTS = [
    cfg.StrOpt('lockname',
               default='/var/lock/ahc-match.lock',
               help='Lock file for edeploy. (Must be writable by the user '
                    'doing matching)'),
    cfg.StrOpt('configdir',
               default='/etc/edeploy',
               help='Directory containing the edeploy state, .specs and .cmdb '
                    'files. (Must be writable by the user doing matching)'),
]
MATCH_OPTS = [
    cfg.BoolOpt('debug',
                default=False,
                help='Debug mode enabled/disabled.')
]
cfg.CONF.register_opts(EDEPLOY_OPTS, group='edeploy')
cfg.CONF.register_opts(MATCH_OPTS, group='match')


def list_opts():
    return [
        ('match', MATCH_OPTS),
        ('edeploy', EDEPLOY_OPTS)
    ]



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import unittest

from hardware import cmdb
from hardware import state
import mock
from oslo_config import cfg

from ahc_tools import exc
from ahc_tools import match
from ahc_tools import utils

CONF = cfg.CONF


def fake_load(obj, cfg_dir):
    obj._cfg_dir = cfg_dir
    obj._data = [('hw1', '*'), ]


class TestBase(unittest.TestCase):
    def setUp(self):
        super(TestBase, self).setUp()
        basedir = os.path.dirname(os.path.abspath(__file__))
        CONF.set_override('configdir',
                          os.path.join(basedir, 'edeploy_conf'),
                          'edeploy')
        self.uuid = '1a1a1a1a-2b2b-3c3c-4d4d-5e5e5e5e5e5e'
        self.bmc_address = '1.2.3.4'
        self.node = mock.Mock(driver='pxe_ipmitool',
                              driver_info={'ipmi_address': self.bmc_address},
                              properties={'cpu_arch': 'i386', 'local_gb': 40},
                              uuid=self.uuid,
                              power_state='power on',
                              provision_state='inspecting',
                              extra={'on_discovery': 'true'},
                              instance_uuid=None,
                              maintenance=False)
        self.facts = [
            ['network', 'eth0', 'serial', '99:99:99:99:99:99'],
            ['network', 'eth0', 'ipv4', '192.168.100.12']]


@mock.patch.object(utils, 'get_facts', autospec=True)
@mock.patch.object(state.State, 'load', fake_load)
@mock.patch.object(state.State, '_load_specs',
                   lambda o, n: [('network', '$iface', 'serial', '$mac'),
                                 ('network', '$iface', 'ipv4', '$ipv4')])
class TestMatch(TestBase):
    def setUp(self):
        super(TestMatch, self).setUp()

    def test_match(self, mock_facts):
        mock_facts.return_value = self.facts
        node_info = {}
        match.match(self.node, node_info)
        self.assertEqual('hw1', node_info['hardware']['profile'])
        self.assertEqual('eth0', node_info['hardware']['iface'])
        self.assertEqual('99:99:99:99:99:99', node_info['hardware']['mac'])
        self.assertEqual('192.168.100.12', node_info['hardware']['ipv4'])

        node_patches = match.get_update_patches(self.node, node_info)
        self.assertEqual('/extra/configdrive_metadata',
                         node_patches[0]['path'])
        self.assertEqual('hw1',
                         node_patches[0]['value']['hardware']['profile'])
        self.assertEqual('/properties/capabilities',
                         node_patches[1]['path'])
        self.assertEqual('profile:hw1',
                         node_patches[1]['value'])

    @mock.patch.object(state.State, '__init__',
                       side_effect=Exception('boom'), autospec=True)
    def test_load_failed(self, state_mock, mock_facts):
        self.assertRaises(exc.LoadFailedError, match.match,
                          self.node, self.facts)

    @mock.patch.object(state.State, 'find_match',
                       side_effect=Exception('boom'), autospec=True)
    def test_no_match(self, find_mock, mock_facts):
        self.assertRaises(exc.MatchFailedError, match.match, self.node, {})

    def test_multiple_capabilities(self, mock_facts):
        self.node.properties['capabilities'] = 'cat:meow,profile:robin'
        node_info = {'hardware': {'profile': 'batman'}, 'edeploy_facts': []}
        node_patches = match.get_update_patches(self.node, node_info)
        self.assertIn('cat:meow', node_patches[1]['value'])
        self.assertIn('profile:batman', node_patches[1]['value'])
        # Assert the old profile is gone
        self.assertNotIn('profile:robin', node_patches[1]['value'])

    def test_no_data(self, mock_facts):
        node_info = {}
        self.assertEqual([], match.get_update_patches(self.node, node_info))

    @mock.patch.object(cmdb, 'load_cmdb')
    def test_raid_configuration_passed(self, mock_load_cmdb, mock_facts):
        mock_load_cmdb.return_value = [
            {'logical_disks': (
                {'disk_type': 'hdd',
                 'interface_type': 'sas',
                 'is_root_volume': 'true',
                 'raid_level': '1+0',
                 'size_gb': 50,
                 'volume_name': 'root_volume'},
                {'disk_type': 'hdd',
                 'interface_type': 'sas',
                 'number_of_physical_disks': 3,
                 'raid_level': '5',
                 'size_gb': 100,
                 'volume_name': 'data_volume'})}]
        mock_facts.return_value = self.facts
        node_info = {}
        match.match(self.node, node_info)
        self.assertIn('target_raid_configuration', node_info)

        node_patches = match.get_update_patches(self.node, node_info)
        self.assertEqual('/extra/target_raid_configuration',
                         node_patches[2]['path'])

    @mock.patch.object(cmdb, 'load_cmdb')
    def test_bios_configuration_passed(self, mock_load_cmdb, mock_facts):
        mock_load_cmdb.return_value = [
            {'bios_settings': {'ProcVirtualization': 'Disabled'}}]
        mock_facts.return_value = self.facts
        node_info = {}
        match.match(self.node, node_info)
        self.assertIn('bios_settings', node_info)

        node_patches = match.get_update_patches(self.node, node_info)
        self.assertEqual('/extra/bios_settings',
                         node_patches[2]['path'])


@mock.patch.object(match, 'LOG')
@mock.patch.object(utils, 'get_ironic_client', autospec=True)
@mock.patch.object(match, '_restore_state', lambda: None)
class TestMain(TestBase):
    def setUp(self):
        super(TestMain, self).setUp()
        self.mock_client = mock.Mock()
        self.mock_client.node.list.return_value = [self.node]

    @mock.patch.object(match, '_copy_state', side_effect=Exception('boom'))
    def test_copy_failed(self, mock_copy, mock_ic, mock_log):
        mock_ic.return_value = self.mock_client
        self.assertRaises(SystemExit, match.main)
        self.assertTrue(1, mock_log.error.call_count)

    @mock.patch.object(match, 'match', autospec=True,
                       side_effect=exc.LoadFailedError('boom', '/etc/edeploy'))
    @mock.patch.object(match, '_copy_state', lambda: None)
    def test_load_failed(self, mock_match, mock_ic, mock_log):
        mock_ic.return_value = self.mock_client
        self.assertRaises(SystemExit, match.main)
        self.assertTrue(1, mock_log.error.call_count)

    @mock.patch.object(match, 'get_update_patches', autospec=True)
    @mock.patch.object(match, 'match', autospec=True,
                       side_effect=exc.MatchFailedError('boom', '1234567890'))
    @mock.patch.object(match, '_copy_state', lambda: None)
    def test_match_failed(self, mock_match, mock_update, mock_ic, mock_log):
        mock_ic.return_value = self.mock_client
        match.main()
        self.assertEqual(2, mock_log.error.call_count)
        self.assertFalse(mock_update.called)

    @mock.patch.object(match, 'match', lambda x, y: None)
    @mock.patch.object(match, 'get_update_patches', lambda x, y: None)
    @mock.patch.object(match, '_copy_state', lambda: None)
    def test_match_success(self, mock_ic, mock_log):
        mock_ic.return_value = self.mock_client
        match.main()
        self.assertFalse(mock_log.error.called)

    @mock.patch.object(match, 'match', lambda x, y: None)
    @mock.patch.object(match, 'get_update_patches', lambda x, y: None)
    @mock.patch.object(match, '_copy_state', lambda: None)
    def test_update_failed(self, mock_ic, mock_log):
        self.mock_client.node.update.side_effect = Exception('boom')
        mock_ic.return_value = self.mock_client
        match.main()
        self.assertTrue(1, mock_log.error.call_count)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys
import unittest

from ahc_tools import report

from hardware.cardiff import cardiff
from hardware.cardiff import compare_sets
from hardware.cardiff import utils
import mock


@mock.patch.object(utils, 'get_hosts_list', autospec=True)
@mock.patch.object(compare_sets, 'print_systems_groups', autospec=True)
@mock.patch.object(cardiff, 'group_systems', autospec=True)
@mock.patch.object(cardiff, 'compare_performance', autospec=True)
class TestPrintReport(unittest.TestCase):
    def setUp(self):
        self.args = mock.Mock()
        self.args.groups = False
        self.args.full = False
        self.args.categories = False
        self.args.outliers = False
        self.args.unique_id = 'uuid'
        self.facts = []
        self.detail = {'group': '', 'category': '', 'item': ''}

    def test_groups(self, cp_mock, gs_mock, psg_mock, ghl_mock):
        self.args.groups = True
        ghl_mock.return_value = []
        report.print_report(self.args, self.facts)
        ghl_mock.assert_called_once_with([], 'uuid')
        psg_mock.assert_called_once_with([[]])
        self.assertFalse(cp_mock.called)
        self.assertFalse(gs_mock.called)

    def test_categories(self, cp_mock, gs_mock, psg_mock, ghl_mock):
        self.args.categories = True
        ghl_mock.return_value = []
        report.print_report(self.args, self.facts)
        ghl_mock.assert_called_once_with([], 'uuid')
        gs_mock.assert_called_once_with({}, self.facts, 'uuid', [[]], 'system')
        self.assertFalse(cp_mock.called)
        self.assertFalse(psg_mock.called)

    def test_outliers(self, cp_mock, gs_mock, psg_mock, ghl_mock):
        self.args.outliers = True
        ghl_mock.return_value = []
        report.print_report(self.args, self.facts)
        ghl_mock.assert_called_once_with([], 'uuid')
        cp_mock.assert_called_once_with([], 'uuid', [[]], self.detail)
        self.assertFalse(psg_mock.called)
        self.assertFalse(gs_mock.called)

    def test_full(self, cp_mock, gs_mock, psg_mock, ghl_mock):
        self.args.full = True
        ghl_mock.return_value = []
        report.print_report(self.args, self.facts)
        ghl_mock.assert_called_once_with([], 'uuid')
        psg_mock.assert_called_once_with([[]])
        gs_mock.assert_called_once_with({}, self.facts, 'uuid', [[]], 'system')
        cp_mock.assert_called_once_with([], 'uuid', [[]], self.detail)


class TestMain(unittest.TestCase):
    @mock.patch('sys.stdout')
    def test_no_args(self, out_mock):
        sys.argv = ['ahc-report']
        self.assertRaisesRegexp(SystemExit, "1", report.main)

    @mock.patch.object(report.utils, 'get_ironic_client', autospec=True)
    @mock.patch.object(report.utils, 'get_facts', autospec=True)
    @mock.patch.object(report, 'print_report', autospec=True)
    def test_no_exceptions(self, print_mock, facts_mock, ic_mock):
        with mock.patch('sys.argv', ['ahc-report', '-f']):
            report.main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

from ironicclient import client
from ironicclient.exc import AmbiguousAuthSystem


def get_facts(node):
    """Get the facts stored on the Ironic DB"""
    # cardiff expects data in the form of a list of nodes
    # where each node is represented by a list of tuples
    # with each tuple representing a fact about the node
    try:
        facts = [tuple(fact) for fact in node.extra['edeploy_facts']]
    except KeyError:
        err_msg = ("You must run introspection on the nodes before "
                   "running this tool.\n")
        sys.exit(err_msg)
    return facts


def get_ironic_client():
    """Get Ironic client instance."""
    # TODO(trown): refactor to include error handling, and inputing
    #              credentials on the command line. Might also want
    #              getpass for password input.
    kwargs = {'os_password': os.environ.get('OS_PASSWORD'),
              'os_username': os.environ.get('OS_USERNAME'),
              'os_tenant_name': os.environ.get('OS_TENANT_NAME'),
              'os_auth_url': os.environ.get('OS_AUTH_URL')}
    try:
        ironic = client.get_client(1, **kwargs)
    except AmbiguousAuthSystem:
        if kwargs['os_password']:
            kwargs['os_password'] = "<hidden>"
        err_msg = ("Some credentials are missing. The needed environment "
                   "variables are set as follows: "
                   "OS_PASSWORD=%(os_password)s, "
                   "OS_USERNAME=%(os_username)s, "
                   "OS_TENANT_NAME=%(os_tenant_name)s, "
                   "OS_AUTH_URL=%(os_auth_url)s.\n" % kwargs)
        sys.exit(err_msg)
    return ironic


def capabilities_to_dict(caps):
    """Convert the Node's capabilities into a dictionary."""
    if not caps:
        return {}
    return dict(key.split(':', 1) for key in caps.split(','))


def dict_to_capabilities(caps_dict):
    """Convert a dictionary into a string with the capabilities syntax."""
    return ','.join("%s:%s" % tpl for tpl in caps_dict.items())



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import shutil
import sys

from hardware import state
from oslo_config import cfg

from ahc_tools import conf  # noqa
from ahc_tools import exc
from ahc_tools import utils


CONF = cfg.CONF

LOG = logging.getLogger('ahc_tools.match')


def match(node, node_info):
    sobj = None
    try:
        sobj = state.State(lockname=CONF.edeploy.lockname)
        sobj.load(CONF.edeploy.configdir)
    except Exception as e:
        if sobj:
            sobj.unlock()
        raise exc.LoadFailedError(e.__str__(), CONF.edeploy.configdir)

    try:
        facts = utils.get_facts(node)
        profile, data = sobj.find_match(facts)
        data['profile'] = profile

        if 'logical_disks' in data:
            node_info['target_raid_configuration'] = {
                'logical_disks': data.pop('logical_disks')}

        if 'bios_settings' in data:
            node_info['bios_settings'] = data.pop('bios_settings')

        node_info['hardware'] = data
    except Exception as e:
        raise exc.MatchFailedError(e.__str__(), node.uuid)
    finally:
        sobj.save()
        sobj.unlock()


def get_update_patches(node, node_info):
    patches = []

    if 'hardware' not in node_info:
        return []

    capabilities_dict = utils.capabilities_to_dict(
        node.properties.get('capabilities'))
    capabilities_dict['profile'] = node_info['hardware']['profile']

    patches.append({'op': 'add',
                    'path': '/extra/configdrive_metadata',
                    'value': {'hardware': node_info['hardware']}})
    patches.append(
        {'op': 'add',
         'path': '/properties/capabilities',
         'value': utils.dict_to_capabilities(capabilities_dict)})

    if 'target_raid_configuration' in node_info:
        patches.append(
            {'op': 'add',
             'path': '/extra/target_raid_configuration',
             'value': node_info['target_raid_configuration']})

    if 'bios_settings' in node_info:
        patches.append(
            {'op': 'add',
             'path': '/extra/bios_settings',
             'value': node_info['bios_settings']})

    return patches


def main():
    CONF(sys.argv[1:])
    debug = CONF.match.debug
    logging.basicConfig(level=logging.DEBUG if debug else logging.INFO)
    for third_party in ('urllib3.connectionpool',
                        'keystoneclient.auth',
                        'iso8601.iso8601',
                        'requests.packages.urllib3.connectionpool'):
        logging.getLogger(third_party).setLevel(logging.WARNING)
    logging.getLogger('ironicclient.common.http').setLevel(
        logging.INFO if debug else logging.ERROR)
    logging.getLogger('hardware.state').setLevel(
        logging.INFO if debug else logging.WARNING)

    ironic_client = utils.get_ironic_client()
    nodes = ironic_client.node.list(detail=True)
    patches = {}

    try:
        _copy_state()
    except Exception as e:
        err_msg = ('Failed to copy the state file: %s/state. '
                   'Error was: %s' % (CONF.edeploy.configdir, e.__str__()))
        LOG.error(err_msg)
        sys.exit()

    failed_nodes = []
    for node in nodes:
        try:
            node_info = {}
            LOG.debug('Attempting to match node %s' % node.uuid)
            match(node, node_info)
            patches[node.uuid] = get_update_patches(node, node_info)
        except exc.LoadFailedError as e:
            LOG.error(e.message)
            sys.exit()
        except exc.MatchFailedError as e:
            LOG.error(e.message)
            failed_nodes.append(node)

    if failed_nodes:
        err_msg = ('The following nodes did not match any profiles '
                   'and will not be updated: ' +
                   ','.join(node.uuid for node in failed_nodes))
        LOG.error(err_msg)
    _restore_state()

    for node in nodes:
        if node not in failed_nodes:
            try:
                ironic_client.node.update(node.uuid, patches[node.uuid])
            except Exception as e:
                err_msg = ('Failed to update node (%s). '
                           'Error was: %s' % (node.uuid, e.__str__()))
                LOG.error(err_msg)


def _copy_state():
    src = CONF.edeploy.configdir + '/state'
    dst = CONF.edeploy.configdir + '/state.bak'
    shutil.copyfile(src, dst)


def _restore_state():
    src = CONF.edeploy.configdir + '/state.bak'
    dst = CONF.edeploy.configdir + '/state'
    shutil.copyfile(src, dst)
